package com.dragon.system.server.controller;

import com.dragon.common.core.domain.Result;
import com.dragon.common.core.utils.poi.ExcelUtil;
import com.dragon.common.core.web.controller.BaseController;
import com.dragon.common.core.web.page.TableDataInfo;
import com.dragon.common.log.annotation.Log;
import com.dragon.common.log.enums.BusinessType;
import com.dragon.common.security.annotation.RequiresPermissions;
import com.dragon.common.security.utils.SecurityUtils;
import com.dragon.system.common.domain.SysConfig;
import com.dragon.system.server.service.SysConfigPlusService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.List;

;

/**
 * 参数配置 信息操作处理
 *
 * @author dragon
 */
@RestController
@RequestMapping("/config/plus")
public class SysConfigPlusController extends BaseController {
    @Autowired
    private SysConfigPlusService configService;

    /**
     * 分页获取系统服务参数配置列表
     */
    @RequiresPermissions("system:config:list")
    @GetMapping("/list")
    public Result<TableDataInfo<SysConfig>> list(SysConfig config) {
        startPage();
        List<SysConfig> list = configService.pageQuery(config);
        return getDataTable(list);
    }

    @Log(title = "参数管理", businessType = BusinessType.EXPORT)
    @RequiresPermissions("system:config:export")
    @PostMapping("/export")
    public void export(HttpServletResponse response, SysConfig config) {
        List<SysConfig> list = configService.pageQuery(config);
        ExcelUtil<SysConfig> util = new ExcelUtil<SysConfig>(SysConfig.class);
        util.exportExcel(response, list, "参数数据");
    }

    /**
     * 根据参数编号获取详细信息
     */
    @GetMapping(value = "/{configId}")
    public Result getInfo(@PathVariable Long configId) {
        return Result.success(configService.getById(configId));
    }

    /**
     * 根据参数键名查询参数值
     */
    @GetMapping(value = "/configKey/{configKey}")
    public Result getConfigKey(@PathVariable String configKey) {
        return Result.success(configService.getByConfigKey(configKey));
    }

    /**
     * 新增参数配置
     */
    @RequiresPermissions("system:config:add")
    @Log(title = "参数管理", businessType = BusinessType.INSERT)
    @PostMapping
    public Result add(@Validated @RequestBody SysConfig config) {
        if (!configService.checkConfigKeyUnique(config)) {
            return Result.error("新增参数'" + config.getConfigName() + "'失败，参数键名已存在");
        }
        config.setCreateBy(SecurityUtils.getUsername());
        return toAjax(configService.save(config));
    }

    /**
     * 修改参数配置
     */
    @RequiresPermissions("system:config:edit")
    @Log(title = "参数管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public Result edit(@Validated @RequestBody SysConfig config) {
        if (!configService.checkConfigKeyUnique(config)) {
            return Result.error("修改参数'" + config.getConfigName() + "'失败，参数键名已存在");
        }
        config.setUpdateBy(SecurityUtils.getUsername());
        return toAjax(configService.updateById(config));
    }

    /**
     * 删除参数配置
     */
    @RequiresPermissions("system:config:remove")
    @Log(title = "参数管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{configIds}")
    public Result remove(@PathVariable Long[] configIds) {
        configService.removeBatchByIds(Arrays.asList(configIds));
        return success();
    }

    /**
     * 刷新参数缓存
     */
    @RequiresPermissions("system:config:remove")
    @Log(title = "参数管理", businessType = BusinessType.CLEAN)
    @DeleteMapping("/refreshCache")
    public Result refreshCache() {
        configService.resetConfigCache();
        return success();
    }
}
