package com.dragon.system.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dragon.common.core.constant.CacheConstants;
import com.dragon.common.core.utils.StringUtils;
import com.dragon.common.redis.service.RedisService;
import com.dragon.system.common.domain.SysConfig;
import com.dragon.system.server.mapper.SysConfigPlusMapper;
import com.dragon.system.server.service.SysConfigPlusService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Objects;


import static com.dragon.common.security.utils.DictUtils.getCacheKey;

/**
 * @author 冯凯
 * @version 1.0
 * @description: 系统配置业务层实现类
 * @date 2023/11/13 18:53
 */
@Service
public class SysConfigPlusServiceImpl extends ServiceImpl<SysConfigPlusMapper, SysConfig> implements SysConfigPlusService {

    @Autowired
    private RedisService redisService;

    @Resource
    private SysConfigPlusMapper configPlusMapper;
    /**
     * 分页获取系统服务参数配置列表
     */
    @Override
    public List<SysConfig> pageQuery(SysConfig config) {
        LambdaQueryWrapper<SysConfig> sysConfigLambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(config.getConfigName())){
            sysConfigLambdaQueryWrapper.like(SysConfig::getConfigName,config.getConfigName());
        } if (StringUtils.isNotEmpty(config.getConfigKey())){
            sysConfigLambdaQueryWrapper.like(SysConfig::getConfigKey,config.getConfigKey());
        }
        if (StringUtils.isNotEmpty(config.getConfigValue())){
            sysConfigLambdaQueryWrapper.like(SysConfig::getConfigValue,config.getConfigValue());
        }if (StringUtils.isNotEmpty(config.getConfigType())){
            sysConfigLambdaQueryWrapper.like(SysConfig::getConfigType,config.getConfigType());
        }
        //获取时间参数
        Object beginTime = config.getParams().get("beginTime");
        Object endTime = config.getParams().get("endTime");
        if (Objects.nonNull(beginTime) && beginTime instanceof Date beginDate){
            sysConfigLambdaQueryWrapper.gt(SysConfig::getCreateTime,beginDate);
        } if (Objects.nonNull(endTime) && endTime instanceof Date endDate){
            sysConfigLambdaQueryWrapper.lt(SysConfig::getCreateTime,endDate);
        }
        return this.list(sysConfigLambdaQueryWrapper);
    }

    /**
     * 根据参数键名查询参数值
     */
    @Override
    public String getByConfigKey(String configKey) {
        LambdaQueryWrapper<SysConfig> sysConfigLambdaQueryWrapper = new LambdaQueryWrapper<>();
        Assert.notNull(configKey,"configKey不可以为空");
        sysConfigLambdaQueryWrapper.eq(SysConfig::getConfigKey,configKey);
        SysConfig sysConfig = this.getOne(sysConfigLambdaQueryWrapper);
        return sysConfig.getConfigValue();
    }


    /**
     * @description:  检查参数是否唯一
     * @param:  * @param config
     * @return: boolean
     * @author 冯凯
     * @date: 2023/11/13 19:42
     */
    @Override
    public boolean checkConfigKeyUnique(SysConfig config) {
        LambdaQueryWrapper<SysConfig> sysConfigLambdaQueryWrapper = new LambdaQueryWrapper<>();
        Assert.notNull(config.getConfigKey(),"configKey不可以为空");
        sysConfigLambdaQueryWrapper.eq(SysConfig::getConfigKey,config.getConfigKey());
        return  this.count(sysConfigLambdaQueryWrapper)>0;
    }

    /**
     * 刷新参数缓存
     */
    @Override
    public void resetConfigCache() {
        clearConfigCache();
        loadingConfigCache();
    }

    @Override
    public SysConfig selectConfigById(Long configId) {
        SysConfig sysConfig = this.getById(configId);
        return sysConfig;
    }

    /**
     * 清空参数缓存数据
     */
    public void clearConfigCache() {
        Collection<String> keys = redisService.keys(CacheConstants.SYS_CONFIG_KEY + "*");
        redisService.deleteObject(keys);
    }

    /**
     * 加载参数缓存数据
     */
    public void loadingConfigCache() {
        List<SysConfig> configList = this.list();
        for (SysConfig config : configList) {
            redisService.setCacheObject(getCacheKey(config.getConfigKey()), config.getConfigValue());
        }
    }
}
